@@ -14,23 +14,28 @@
 
 #include "snapd-error.h"
 #include "snapd-app.h"
+#include "snapd-category.h"
 #include "snapd-media.h"
 #include "snapd-screenshot.h"
 #include "snapd-task.h"
 
 void
-_snapd_json_set_body (SoupMessage *message, JsonBuilder *builder)
+_snapd_json_set_body (SoupMessage *message, JsonBuilder *builder, GBytes **body)
 {
+#if SOUP_CHECK_VERSION (2, 99, 2)
+    SoupMessageHeaders *request_headers = soup_message_get_request_headers (message);
+#else
+    SoupMessageHeaders *request_headers = message->request_headers;
+#endif
+    soup_message_headers_set_content_type (request_headers, "application/json", NULL);
+
     g_autoptr(JsonNode) json_root = json_builder_get_root (builder);
     g_autoptr(JsonGenerator) json_generator = json_generator_new ();
     json_generator_set_pretty (json_generator, TRUE);
     json_generator_set_root (json_generator, json_root);
     gsize data_length;
     g_autofree guchar *data = (guchar *) json_generator_to_data (json_generator, &data_length);
-
-    soup_message_headers_set_content_type (message->request_headers, "application/json", NULL);
-    soup_message_body_append_take (message->request_body, g_steal_pointer (&data), data_length);
-    soup_message_headers_set_content_length (message->request_headers, message->request_body->length);
+    *body = g_bytes_new_take (g_steal_pointer (&data), data_length);
 }
 
 gboolean
@@ -63,6 +68,19 @@ _snapd_json_get_string (JsonObject *object, const gchar *name, const gchar *defa
         return default_value;
 }
 
+const gchar *
+_snapd_json_get_int_as_string (JsonObject *object, const gchar *name)
+{
+    JsonNode *node = json_object_get_member (object, name);
+    if (node == NULL)
+        return NULL;
+    if (json_node_get_value_type (node) == G_TYPE_INT64)
+        return g_strdup_printf("%ld", json_node_get_int (node));
+    else if (json_node_get_value_type (node) == G_TYPE_STRING)
+        return g_strdup(json_node_get_string (node));
+    return NULL;
+}
+
 JsonArray *
 _snapd_json_get_array (JsonObject *object, const gchar *name)
 {
@@ -83,6 +101,109 @@ _snapd_json_get_object (JsonObject *object, const gchar *name)
         return NULL;
 }
 
+gboolean
+_snapd_json_parse_time_span (const gchar *time_span_string, GTimeSpan *time_span)
+{
+    /* Examples:
+          3y
+          1y5d
+          4h32m2s
+
+       Valid time units: y w d h m s ms us µs ns
+
+       Since GTimeSpan is in microseconds, ns will be discarded.
+     */
+
+    GTimeSpan current_value = 0;
+    if (time_span_string == NULL)
+        return FALSE;
+    gchar *p = (gchar *)time_span_string;
+
+    *time_span = 0;
+    if (*p == '-')
+        p++;
+
+    for (;*p != 0; p++) {
+        switch (*p) {
+        case '0':
+        case '1':
+        case '2':
+        case '3':
+        case '4':
+        case '5':
+        case '6':
+        case '7':
+        case '8':
+        case '9':
+            current_value *= 10;
+            current_value += (GTimeSpan)((*p) - '0');
+            break;
+        case 'y':
+            *time_span += current_value * 365 * G_TIME_SPAN_DAY;
+            current_value = 0;
+            break;
+        case 'w':
+            *time_span += current_value * 7 * G_TIME_SPAN_DAY;
+            current_value = 0;
+            break;
+        case 'd':
+            *time_span += current_value * G_TIME_SPAN_DAY;
+            current_value = 0;
+            break;
+        case 'h':
+            *time_span += current_value * G_TIME_SPAN_HOUR;
+            current_value = 0;
+            break;
+        case 'm':
+            if (*(p+1) == 's') {
+                *time_span += current_value * G_TIME_SPAN_MILLISECOND;
+                p++;
+            } else {
+                *time_span += current_value * G_TIME_SPAN_MINUTE;
+            }
+            current_value = 0;
+            break;
+        case 's':
+            *time_span += current_value * G_TIME_SPAN_SECOND;
+            current_value = 0;
+            break;
+        case '\xc2': // first byte of µ
+            p++;
+            if (*p != '\xb5') { // it's not µ
+                *time_span = 0;
+                return FALSE;
+            }
+            // continue like it is 'u'
+        case 'u':
+            p++;
+            if (*p == 's') {
+                *time_span += current_value; // microseconds
+                current_value = 0;
+            } else {
+                *time_span = 0;
+                return FALSE;
+            }
+            break;
+        case 'n':
+            p++;
+            if (*p == 's') {
+                current_value = 0;
+            } else {
+                *time_span = 0;
+                return FALSE;
+            }
+            break;
+        default:
+            *time_span = 0;
+            return FALSE;
+        }
+    }
+    if (*time_span_string == '-') {
+        *time_span *= -1;
+    }
+    return TRUE;
+}
+
 static gboolean
 parse_date (const gchar *date_string, gint *year, gint *month, gint *day)
 {
@@ -326,6 +447,26 @@ parse_error_response (JsonObject *root, JsonNode **error_value, GError **error)
                              SNAPD_ERROR,
                              SNAPD_ERROR_UNSUCCESSFUL,
                              message);
+    else if (g_strcmp0 (kind, "app-not-found") == 0)
+        g_set_error_literal (error,
+                             SNAPD_ERROR,
+                             SNAPD_ERROR_APP_NOT_FOUND,
+                             message);
+    else if (g_strcmp0 (kind, "snap-architecture-not-available") == 0)
+         g_set_error_literal (error,
+                              SNAPD_ERROR,
+                              SNAPD_ERROR_ARCHITECTURE_NOT_AVAILABLE,
+                              message);
+    else if (g_strcmp0 (kind, "snap-change-conflict") == 0)
+         g_set_error_literal (error,
+                              SNAPD_ERROR,
+                              SNAPD_ERROR_CHANGE_CONFLICT,
+                              message);
+    else if (g_strcmp0 (kind, "interfaces-unchanged") == 0)
+         g_set_error_literal (error,
+                              SNAPD_ERROR,
+                              SNAPD_ERROR_INTERFACES_UNCHANGED,
+                              message);
     else {
         switch (status_code) {
         case SOUP_STATUS_BAD_REQUEST:
@@ -367,9 +508,8 @@ parse_error_response (JsonObject *root, JsonNode **error_value, GError **error)
 }
 
 JsonObject *
-_snapd_json_parse_response (SoupMessage *message, SnapdMaintenance **maintenance, JsonNode **error_value, GError **error)
+_snapd_json_parse_response (const gchar *content_type, GBytes *body, SnapdMaintenance **maintenance, JsonNode **error_value, GError **error)
 {
-    const gchar *content_type = soup_message_headers_get_content_type (message->response_headers, NULL);
     if (content_type == NULL) {
         g_set_error (error, SNAPD_ERROR, SNAPD_ERROR_BAD_RESPONSE, "snapd returned no content type");
         return NULL;
@@ -380,9 +520,8 @@ _snapd_json_parse_response (SoupMessage *message, SnapdMaintenance **maintenance
     }
 
     g_autoptr(JsonParser) parser = json_parser_new ();
-    g_autoptr(SoupBuffer) buffer = soup_message_body_flatten (message->response_body);
     g_autoptr(GError) error_local = NULL;
-    if (!json_parser_load_from_data (parser, buffer->data, buffer->length, &error_local)) {
+    if (!json_parser_load_from_data (parser, g_bytes_get_data (body, NULL), g_bytes_get_size (body), &error_local)) {
         g_set_error (error, SNAPD_ERROR, SNAPD_ERROR_BAD_RESPONSE, "Unable to parse snapd response: %s", error_local->message);
         return NULL;
     }
@@ -498,6 +637,102 @@ _snapd_json_get_async_result (JsonObject *response, GError **error)
     return g_strdup (json_node_get_string (change_node));
 }
 
+static GStrv
+create_str_array_from_jsonarray (JsonArray *data) {
+    if (data == NULL)
+        return NULL;
+
+    guint len = json_array_get_length (data);
+    GStrv list = g_malloc0_n (1 + len, sizeof(gchar *));
+
+    for (guint i = 0; i <len; i++) {
+        JsonNode *node = json_array_get_element (data, i);
+        if (node == NULL || json_node_get_value_type (node) != G_TYPE_STRING) {
+            g_strfreev (list);
+            return NULL;
+        }
+        list[i] = g_strdup(json_node_get_string (node));
+    }
+    return list;
+}
+
+static void
+add_notice_to_list (JsonArray *array, guint index, JsonNode *element, void *data)
+{
+    GPtrArray *list = (GPtrArray *) data;
+    GTimeSpan expire_after = 0;
+    GTimeSpan repeat_after = 0;
+    g_autoptr(GHashTable) last_data = NULL;
+
+    JsonObject *object = json_node_get_object (element);
+    g_autoptr(GDateTime) first_occurred = _snapd_json_get_date_time (object, "first-occurred");
+    g_autoptr(GDateTime) last_occurred = _snapd_json_get_date_time (object, "last-occurred");
+    g_autoptr(GDateTime) last_repeated = _snapd_json_get_date_time (object, "last-repeated");
+
+    _snapd_json_parse_time_span (_snapd_json_get_string (object, "expire-after", NULL), &expire_after);
+    _snapd_json_parse_time_span (_snapd_json_get_string (object, "repeat-after", NULL), &repeat_after);
+
+    const gchar *notice_type_string = _snapd_json_get_string (object, "type", NULL);
+
+    SnapdNoticeType notice_type;
+    if (notice_type_string == NULL)
+        notice_type = SNAPD_NOTICE_TYPE_UNKNOWN;
+    else if (g_str_equal (notice_type_string, "change-update"))
+        notice_type = SNAPD_NOTICE_TYPE_CHANGE_UPDATE;
+    else if (g_str_equal (notice_type_string, "refresh-inhibit"))
+        notice_type = SNAPD_NOTICE_TYPE_REFRESH_INHIBIT;
+    else if (g_str_equal (notice_type_string, "snap-run-inhibit"))
+        notice_type = SNAPD_NOTICE_TYPE_SNAP_RUN_INHIBIT;
+    else
+        notice_type = SNAPD_NOTICE_TYPE_UNKNOWN;
+
+    if (json_object_has_member (object, "last-data")) {
+        JsonObjectIter iter;
+        const gchar *member_name;
+        JsonNode *member_node;
+
+        last_data = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+        JsonObject *data_dict = _snapd_json_get_object (object, "last-data");
+
+        json_object_iter_init (&iter, data_dict);
+        while (json_object_iter_next (&iter, &member_name, &member_node))
+            g_hash_table_insert (last_data, g_strdup((gchar *)member_name), g_strdup((gchar *)json_node_get_string (member_node)));
+    }
+    g_autofree const gchar *user_id = _snapd_json_get_int_as_string (object, "user-id");
+    g_autoptr(SnapdNotice) notice = g_object_new (SNAPD_TYPE_NOTICE,
+                                                  "id", _snapd_json_get_string (object, "id", NULL),
+                                                  "user-id", user_id,
+                                                  "notice-type", notice_type,
+                                                  "key", _snapd_json_get_string (object, "key", NULL),
+                                                  "first-occurred", first_occurred,
+                                                  "last-occurred", last_occurred,
+                                                  "last-repeated", last_repeated,
+                                                  "occurrences", _snapd_json_get_int (object, "occurrences", -1),
+                                                  "expire-after", expire_after,
+                                                  "repeat-after", repeat_after,
+                                                  "last-data", last_data,
+                                                  NULL);
+
+    g_ptr_array_add (list, g_object_ref(notice));
+}
+
+GPtrArray *
+_snapd_json_parse_notice (JsonNode *node, GError **error)
+{
+    if (json_node_get_value_type (node) != JSON_TYPE_ARRAY) {
+        g_set_error (error,
+                     SNAPD_ERROR,
+                     SNAPD_ERROR_READ_FAILED,
+                     "Unexpected change type");
+        return NULL;
+    }
+    JsonArray *notices = json_node_get_array (node);
+    GPtrArray *retlist = g_ptr_array_new_full(json_array_get_length (notices), g_object_unref);
+    json_array_foreach_element (notices, add_notice_to_list, retlist);
+
+    return retlist;
+}
+
 SnapdChange *
 _snapd_json_parse_change (JsonNode *node, GError **error)
 {
@@ -544,6 +779,27 @@ _snapd_json_parse_change (JsonNode *node, GError **error)
     g_autoptr(GDateTime) main_spawn_time = _snapd_json_get_date_time (object, "spawn-time");
     g_autoptr(GDateTime) main_ready_time = _snapd_json_get_date_time (object, "ready-time");
 
+    g_autoptr(SnapdChangeData) data = NULL;
+    JsonObject *autorefresh_data = _snapd_json_get_object (object, "data");
+    const gchar *kind = _snapd_json_get_string (object, "kind", "");
+
+    // Currently, only one kind of Change can have DATA field, so this is quite "hardcoded". When more
+    // Change kinds have DATA field, new subclasses of SNAPD_TYPE_CHANGE_DATA must be created, and
+    // the code in QT's snapd-change-data.c file must be completed to manage the new types.
+    if (g_str_equal (kind, "auto-refresh") && (autorefresh_data != NULL)) {
+
+        g_autoptr(JsonArray) snap_names_json = _snapd_json_get_array (autorefresh_data, "snap-names");
+        g_auto(GStrv) snap_names = create_str_array_from_jsonarray (snap_names_json);
+
+        g_autoptr(JsonArray) refresh_forced_json = _snapd_json_get_array (autorefresh_data, "refresh-forced");
+        g_auto(GStrv) refresh_forced = create_str_array_from_jsonarray (refresh_forced_json);
+
+        data = g_object_new (SNAPD_TYPE_AUTOREFRESH_CHANGE_DATA,
+                             "snap-names", snap_names,
+                             "refresh-forced", refresh_forced,
+                             NULL);
+    }
+
     return g_object_new (SNAPD_TYPE_CHANGE,
                          "id", _snapd_json_get_string (object, "id", NULL),
                          "kind", _snapd_json_get_string (object, "kind", NULL),
@@ -554,6 +810,7 @@ _snapd_json_parse_change (JsonNode *node, GError **error)
                          "spawn-time", main_spawn_time,
                          "ready-time", main_ready_time,
                          "error", _snapd_json_get_string (object, "err", NULL),
+                         "data", data,
                          NULL);
 }
 
@@ -582,6 +839,7 @@ _snapd_json_parse_system_information (JsonNode *node, GError **error)
     }
     JsonObject *object = json_node_get_object (node);
 
+    const gchar *architecture = _snapd_json_get_string (object, "architecture", "");
     const gchar *confinement_string = _snapd_json_get_string (object, "confinement", "");
     SnapdSystemConfinement confinement = SNAPD_SYSTEM_CONFINEMENT_UNKNOWN;
     if (strcmp (confinement_string, "strict") == 0)
@@ -629,6 +887,7 @@ _snapd_json_parse_system_information (JsonNode *node, GError **error)
     g_autoptr(GDateTime) refresh_next = _snapd_json_get_date_time (refresh, "next");
 
     return g_object_new (SNAPD_TYPE_SYSTEM_INFORMATION,
+                         "architecture", architecture,
                          "binaries-directory", locations != NULL ? _snapd_json_get_string (locations, "snap-bin-dir", NULL) : NULL,
                          "build-id", _snapd_json_get_string (object, "build-id", NULL),
                          "confinement", confinement,
@@ -706,6 +965,24 @@ _snapd_json_parse_snap (JsonNode *node, GError **error)
         g_ptr_array_add (apps_array, app);
     }
 
+    g_autoptr(JsonArray) categories = _snapd_json_get_array (object, "categories");
+    g_autoptr(GPtrArray) categories_array = g_ptr_array_new_with_free_func (g_object_unref);
+    for (guint i = 0; i < json_array_get_length (categories); i++) {
+        JsonNode *node = json_array_get_element (categories, i);
+
+        if (json_node_get_value_type (node) != JSON_TYPE_OBJECT) {
+            g_set_error (error, SNAPD_ERROR, SNAPD_ERROR_READ_FAILED, "Unexpected categories type");
+            return NULL;
+        }
+
+        JsonObject *c = json_node_get_object (node);
+        g_autoptr(SnapdCategory) category = g_object_new (SNAPD_TYPE_CATEGORY,
+                                                          "featured", _snapd_json_get_bool (c, "featured", FALSE),
+                                                          "name", _snapd_json_get_string (c, "name", NULL),
+                                                          NULL);
+        g_ptr_array_add (categories_array, g_steal_pointer (&category));
+    }
+
     JsonObject *channels = _snapd_json_get_object (object, "channels");
     g_autoptr(GPtrArray) channels_array = g_ptr_array_new_with_free_func (g_object_unref);
     if (channels != NULL) {
@@ -751,6 +1028,7 @@ _snapd_json_parse_snap (JsonNode *node, GError **error)
     g_ptr_array_add (common_ids_array, NULL);
 
     g_autoptr(GDateTime) install_date = _snapd_json_get_date_time (object, "install-date");
+    g_autoptr(GDateTime) hold = _snapd_json_get_date_time (object, "hold");
 
     JsonObject *prices = _snapd_json_get_object (object, "prices");
     g_autoptr(GPtrArray) prices_array = g_ptr_array_new_with_free_func (g_object_unref);
@@ -825,21 +1103,26 @@ _snapd_json_parse_snap (JsonNode *node, GError **error)
         publisher_id = _snapd_json_get_string (publisher, "id", NULL);
         publisher_username = _snapd_json_get_string (publisher, "username", publisher_username);
         const gchar *validation = _snapd_json_get_string (publisher, "validation", NULL);
-        if (validation == NULL || g_strcmp0 (validation, "") == 0)
-            publisher_validation = SNAPD_PUBLISHER_VALIDATION_UNKNOWN;
-        else if (g_strcmp0 (validation, "unproven") == 0)
+        if (g_strcmp0 (validation, "unproven") == 0)
             publisher_validation = SNAPD_PUBLISHER_VALIDATION_UNPROVEN;
+        else if (g_strcmp0 (validation, "starred") == 0)
+            publisher_validation = SNAPD_PUBLISHER_VALIDATION_STARRED;
         else if (g_strcmp0 (validation, "verified") == 0)
             publisher_validation = SNAPD_PUBLISHER_VALIDATION_VERIFIED;
-        /* Any unknown validation is treated as verified for forwards compatibility */
         else
-            publisher_validation = SNAPD_PUBLISHER_VALIDATION_VERIFIED;
+            publisher_validation = SNAPD_PUBLISHER_VALIDATION_UNKNOWN;
     }
 
+    JsonObject *refresh_inhibit = _snapd_json_get_object (object, "refresh-inhibit");
+    g_autoptr(GDateTime) proceed_time = NULL;
+    if (refresh_inhibit != NULL)
+        proceed_time = _snapd_json_get_date_time (refresh_inhibit, "proceed-time");
+
     return g_object_new (SNAPD_TYPE_SNAP,
                          "apps", apps_array,
                          "base", _snapd_json_get_string (object, "base", NULL),
                          "broken", _snapd_json_get_string (object, "broken", NULL),
+                         "categories", categories_array,
                          "channel", _snapd_json_get_string (object, "channel", NULL),
                          "channels", channels_array,
                          "common-ids", (GStrv) common_ids_array->pdata,
@@ -848,6 +1131,7 @@ _snapd_json_parse_snap (JsonNode *node, GError **error)
                          "description", _snapd_json_get_string (object, "description", NULL),
                          "devmode", _snapd_json_get_bool (object, "devmode", FALSE),
                          "download-size", _snapd_json_get_int (object, "download-size", 0),
+                         "hold", hold,
                          "icon", _snapd_json_get_string (object, "icon", NULL),
                          "id", _snapd_json_get_string (object, "id", NULL),
                          "install-date", install_date,
@@ -875,6 +1159,7 @@ _snapd_json_parse_snap (JsonNode *node, GError **error)
                          "trymode", _snapd_json_get_bool (object, "trymode", FALSE),
                          "version", _snapd_json_get_string (object, "version", NULL),
                          "website", _snapd_json_get_string (object, "website", NULL),
+                         "proceed-time", proceed_time,
                          NULL);
 }
 
