@@ -7,8 +7,7 @@
  * See http://www.gnu.org/copyleft/lgpl.html the full text of the license.
  */
 
-#ifndef __SNAPD_JSON_H__
-#define __SNAPD_JSON_H__
+#pragma once
 
 #include <libsoup/soup.h>
 #include <json-glib/json-glib.h>
@@ -19,6 +18,7 @@
 #include "snapd-connection.h"
 #include "snapd-interface.h"
 #include "snapd-maintenance.h"
+#include "snapd-notice.h"
 #include "snapd-plug.h"
 #include "snapd-plug-ref.h"
 #include "snapd-slot.h"
@@ -30,7 +30,8 @@
 G_BEGIN_DECLS
 
 void                  _snapd_json_set_body               (SoupMessage        *message,
-                                                          JsonBuilder        *builder);
+                                                          JsonBuilder        *builder,
+                                                          GBytes            **body);
 
 gboolean              _snapd_json_get_bool               (JsonObject         *object,
                                                           const gchar        *name,
@@ -53,7 +54,8 @@ JsonObject           *_snapd_json_get_object             (JsonObject         *ob
 GDateTime            *_snapd_json_get_date_time          (JsonObject         *object,
                                                           const gchar        *name);
 
-JsonObject           *_snapd_json_parse_response         (SoupMessage        *message,
+JsonObject           *_snapd_json_parse_response         (const gchar        *content_type,
+                                                          GBytes             *body,
                                                           SnapdMaintenance **maintenance,
                                                           JsonNode          **error_value,
                                                           GError            **error);
@@ -70,6 +72,9 @@ JsonArray            *_snapd_json_get_sync_result_a      (JsonObject         *re
 gchar                *_snapd_json_get_async_result       (JsonObject         *response,
                                                           GError            **error);
 
+GPtrArray            *_snapd_json_parse_notice           (JsonNode            *node,
+                                                          GError             **error);
+
 SnapdChange          *_snapd_json_parse_change           (JsonNode            *node,
                                                           GError            **error);
 
@@ -116,5 +121,3 @@ SnapdInterface       *_snapd_json_parse_interface        (JsonNode           *no
                                                           GError            **error);
 
 G_END_DECLS
-
-#endif /* __SNAPD_JSON_H__ */
