@@ -16,10 +16,12 @@
 #include <QLocalSocket>
 #include <Snapd/Alias>
 #include <Snapd/AuthData>
+#include <Snapd/CategoryDetails>
 #include <Snapd/Connection>
 #include <Snapd/Icon>
 #include <Snapd/Interface>
 #include <Snapd/Maintenance>
+#include <Snapd/Notice>
 #include <Snapd/Plug>
 #include <Snapd/Request>
 #include <Snapd/Slot>
@@ -455,7 +457,7 @@ class Q_DECL_EXPORT QSnapdFindRequest : public QSnapdRequest
     Q_PROPERTY(QString suggestedCurrency READ suggestedCurrency)
 
 public:
-    explicit QSnapdFindRequest (int flags, const QString& section, const QString& name, void *snapd_client, QObject *parent = 0);
+    explicit QSnapdFindRequest (int flags, const QString& section, const QString& category, const QString& name, void *snapd_client, QObject *parent = 0);
     ~QSnapdFindRequest ();
     virtual void runSync ();
     virtual void runAsync ();
@@ -739,6 +741,25 @@ private:
     Q_DECLARE_PRIVATE(QSnapdGetSectionsRequest)
 };
 
+class QSnapdGetCategoriesRequestPrivate;
+class Q_DECL_EXPORT QSnapdGetCategoriesRequest : public QSnapdRequest
+{
+    Q_OBJECT
+
+public:
+    explicit QSnapdGetCategoriesRequest (void *snapd_client, QObject *parent = 0);
+    ~QSnapdGetCategoriesRequest ();
+    virtual void runSync ();
+    virtual void runAsync ();
+    Q_INVOKABLE int categoryCount () const;
+    Q_INVOKABLE QSnapdCategoryDetails *category (int) const;
+    void handleResult (void *, void *);
+
+private:
+    QScopedPointer<QSnapdGetCategoriesRequestPrivate> d_ptr;
+    Q_DECLARE_PRIVATE(QSnapdGetCategoriesRequest)
+};
+
 class QSnapdGetAliasesRequestPrivate;
 class Q_DECL_EXPORT QSnapdGetAliasesRequest : public QSnapdRequest
 {
@@ -899,6 +920,85 @@ private:
     Q_DECLARE_PRIVATE(QSnapdDownloadRequest)
 };
 
+class QSnapdCheckThemesRequestPrivate;
+class Q_DECL_EXPORT QSnapdCheckThemesRequest : public QSnapdRequest
+{
+    Q_OBJECT
+
+public:
+    enum ThemeStatus
+    {
+       ThemeInstalled,
+       ThemeAvailable,
+       ThemeUnavailable
+    };
+    Q_ENUM(ThemeStatus)
+
+    explicit QSnapdCheckThemesRequest (const QStringList& gtkThemeNames, const QStringList& iconThemeNames, const QStringList& soundThemeNames, void *snapd_client, QObject *parent = 0);
+    ~QSnapdCheckThemesRequest ();
+    virtual void runSync ();
+    virtual void runAsync ();
+    Q_INVOKABLE ThemeStatus gtkThemeStatus (const QString& name) const;
+    Q_INVOKABLE ThemeStatus iconThemeStatus (const QString& name) const;
+    Q_INVOKABLE ThemeStatus soundThemeStatus (const QString& name) const;
+    void handleResult (void *, void *);
+
+private:
+    QScopedPointer<QSnapdCheckThemesRequestPrivate> d_ptr;
+    Q_DECLARE_PRIVATE(QSnapdCheckThemesRequest)
+};
+
+class QSnapdInstallThemesRequestPrivate;
+class Q_DECL_EXPORT QSnapdInstallThemesRequest : public QSnapdRequest
+{
+    Q_OBJECT
+
+public:
+    explicit QSnapdInstallThemesRequest (const QStringList& gtkThemeNames, const QStringList& iconThemeNames, const QStringList& soundThemeNames, void *snapd_client, QObject *parent = 0);
+    ~QSnapdInstallThemesRequest ();
+    virtual void runSync ();
+    virtual void runAsync ();
+    void handleResult (void *, void *);
+
+private:
+    QScopedPointer<QSnapdInstallThemesRequestPrivate> d_ptr;
+    Q_DECLARE_PRIVATE(QSnapdInstallThemesRequest)
+};
+
+class QSnapdNoticesRequestPrivate;
+class Q_DECL_EXPORT QSnapdNoticesRequest : public QSnapdRequest
+{
+    Q_OBJECT
+    Q_PROPERTY(qint64 timeout MEMBER timeout)
+    Q_PROPERTY(bool sinceFilterSet MEMBER sinceFilterSet)
+    Q_PROPERTY(QDateTime sinceFilter MEMBER sinceFilter)
+    Q_PROPERTY(QString userIdFilter MEMBER userIdFilter)
+    Q_PROPERTY(QString usersFilter MEMBER usersFilter)
+    Q_PROPERTY(QString keysFilter MEMBER keysFilter)
+    Q_PROPERTY(QString typesFilter MEMBER typesFilter)
+
+public:
+    explicit QSnapdNoticesRequest (void *snapd_client, QObject *parent = 0);
+    ~QSnapdNoticesRequest ();
+    virtual void runSync ();
+    virtual void runAsync ();
+    Q_INVOKABLE uint noticesCount () const;
+    Q_INVOKABLE QSnapdNotice *getNotice (quint64) const;
+    void handleResult (void *, void *);
+    void resetFilters ();
+
+private:
+    QScopedPointer<QSnapdNoticesRequestPrivate> d_ptr;
+    Q_DECLARE_PRIVATE(QSnapdNoticesRequest)
+    qint64 timeout;
+    bool sinceFilterSet;
+    QDateTime sinceFilter;
+    QString userIdFilter;
+    QString usersFilter;
+    QString keysFilter;
+    QString typesFilter;
+};
+
 Q_INVOKABLE Q_DECL_DEPRECATED QSnapdLoginRequest *login (const QString& email, const QString& password);
 Q_INVOKABLE Q_DECL_DEPRECATED QSnapdLoginRequest *login (const QString& email, const QString& password, const QString& otp);
 
@@ -923,7 +1023,8 @@ public:
     Q_ENUM(ChangeFilter)
     enum GetSnapsFlag
     {
-        IncludeInactive = 1 << 0
+        IncludeInactive  = 1 << 0,
+        RefreshInhibited = 1 << 1
     };
     Q_DECLARE_FLAGS(GetSnapsFlags, GetSnapsFlag);
     enum GetAppsFlag
@@ -1029,8 +1130,10 @@ public:
     Q_INVOKABLE QSnapdFindRequest *find (const QString &query);
     Q_INVOKABLE QSnapdFindRequest *find (FindFlags flags);
     Q_INVOKABLE QSnapdFindRequest *find (FindFlags flags, const QString &query);
-    Q_INVOKABLE QSnapdFindRequest *findSection (const QString &section, const QString &query);
-    Q_INVOKABLE QSnapdFindRequest *findSection (FindFlags flags, const QString &section, const QString &query);
+    Q_INVOKABLE Q_DECL_DEPRECATED_X("Use findCategory()") QSnapdFindRequest *findSection (const QString &section, const QString &query);
+    Q_INVOKABLE Q_DECL_DEPRECATED_X("Use findCategory()") QSnapdFindRequest *findSection (FindFlags flags, const QString &section, const QString &query);
+    Q_INVOKABLE QSnapdFindRequest *findCategory (const QString &category, const QString &query);
+    Q_INVOKABLE QSnapdFindRequest *findCategory (FindFlags flags, const QString &category, const QString &query);
     Q_INVOKABLE QSnapdFindRefreshableRequest *findRefreshable ();
     Q_INVOKABLE QSnapdInstallRequest *install (const QString &name);
     Q_INVOKABLE QSnapdInstallRequest *install (const QString &name, const QString &channel);
@@ -1055,7 +1158,8 @@ public:
     Q_INVOKABLE QSnapdCreateUserRequest *createUser (const QString& email, CreateUserFlags flags);
     Q_INVOKABLE QSnapdCreateUsersRequest *createUsers ();
     Q_INVOKABLE QSnapdGetUsersRequest *getUsers ();
-    Q_INVOKABLE QSnapdGetSectionsRequest *getSections ();
+    Q_INVOKABLE Q_DECL_DEPRECATED_X("Use getCategories()") QSnapdGetSectionsRequest *getSections ();
+    Q_INVOKABLE QSnapdGetCategoriesRequest *getCategories ();
     Q_INVOKABLE QSnapdGetAliasesRequest *getAliases ();
     Q_INVOKABLE QSnapdAliasRequest *alias (const QString &snap, const QString &app, const QString &alias);
     Q_INVOKABLE QSnapdUnaliasRequest *unalias (const QString &snap, const QString &alias);
@@ -1067,6 +1171,9 @@ public:
     Q_INVOKABLE QSnapdRunSnapCtlRequest *runSnapCtl (const QString contextId, const QStringList &args);
     Q_INVOKABLE QSnapdDownloadRequest *download (const QString &name);
     Q_INVOKABLE QSnapdDownloadRequest *download (const QString &name, const QString &channel, const QString &revision);
+    Q_INVOKABLE QSnapdCheckThemesRequest *checkThemes (const QStringList& gtkThemeNames, const QStringList& iconThemeNames, const QStringList& soundThemeNames);
+    Q_INVOKABLE QSnapdInstallThemesRequest *installThemes (const QStringList& gtkThemeNames, const QStringList& iconThemeNames, const QStringList& soundThemeNames);
+    Q_INVOKABLE QSnapdNoticesRequest *getNotices ();
 
 private:
     QScopedPointer<QSnapdClientPrivate> d_ptr;
