@@ -12,6 +12,7 @@
 
 #include <glib-object.h>
 #include <gio/gio.h>
+#include <json-glib/json-glib.h>
 
 G_BEGIN_DECLS
 
@@ -32,6 +33,7 @@ typedef struct _MockChange MockChange;
 typedef struct _MockChannel MockChannel;
 typedef struct _MockConnection MockConnection;
 typedef struct _MockInterface MockInterface;
+typedef struct _MockLog MockLog;
 typedef struct _MockMedia MockMedia;
 typedef struct _MockPlug MockPlug;
 typedef struct _MockPrice MockPrice;
@@ -40,6 +42,7 @@ typedef struct _MockSnap MockSnap;
 typedef struct _MockSnapshot MockSnapshot;
 typedef struct _MockTask MockTask;
 typedef struct _MockTrack MockTrack;
+typedef struct _MockNotice MockNotice;
 
 MockSnapd      *mock_snapd_new                    (void);
 
@@ -60,6 +63,9 @@ void            mock_snapd_set_maintenance        (MockSnapd     *snapd,
                                                    const gchar   *type,
                                                    const gchar   *message);
 
+void            mock_snapd_set_architecture       (MockSnapd     *snapd,
+                                                   const gchar   *architecture);
+
 void            mock_snapd_set_build_id           (MockSnapd     *snapd,
                                                    const gchar   *build_id);
 
@@ -190,7 +196,7 @@ MockSnap       *mock_snapd_find_snap              (MockSnapd     *snapd,
 MockSnapshot   *mock_snapd_find_snapshot          (MockSnapd     *snapd,
                                                    const gchar   *name);
 
-void            mock_snapd_add_store_section      (MockSnapd     *snapd,
+void            mock_snapd_add_store_category     (MockSnapd     *snapd,
                                                    const gchar   *name);
 
 MockSnap       *mock_snapd_add_store_snap         (MockSnapd     *snapd,
@@ -233,6 +239,10 @@ void            mock_snap_set_base                (MockSnap      *snap,
 void            mock_snap_set_broken              (MockSnap      *snap,
                                                    const gchar   *broken);
 
+void            mock_snap_add_category            (MockSnap    *snap,
+                                                   const gchar *name,
+                                                   gboolean     featured);
+
 void            mock_snap_set_channel             (MockSnap      *snap,
                                                    const gchar   *channel);
 
@@ -306,6 +316,9 @@ void            mock_snap_set_download_size       (MockSnap      *snap,
 void            mock_snap_set_error               (MockSnap      *snap,
                                                    const gchar   *error);
 
+void            mock_snap_set_hold                (MockSnap      *snap,
+                                                   const gchar   *hold);
+
 void            mock_snap_set_icon                (MockSnap      *snap,
                                                    const gchar   *icon);
 
@@ -399,8 +412,9 @@ void            mock_snap_set_version             (MockSnap      *snap,
 void            mock_snap_set_website             (MockSnap      *snap,
                                                    const gchar   *website);
 
-void            mock_snap_add_store_section       (MockSnap      *snap,
-                                                   const gchar   *section);
+void            mock_snap_add_store_category      (MockSnap      *snap,
+                                                   const gchar   *category,
+                                                   gboolean       featured);
 
 MockPlug       *mock_snap_add_plug                (MockSnap      *snap,
                                                    MockInterface *interface,
@@ -459,6 +473,46 @@ void            mock_snapd_set_sound_theme_status (MockSnapd     *snapd,
                                                    const gchar   *name,
                                                    const gchar   *status);
 
+void            mock_snapd_add_log                (MockSnapd     *snapd,
+                                                   const gchar   *timestamp,
+                                                   const gchar   *message,
+                                                   const gchar   *sid,
+                                                   const gchar   *pid);
+
+void            mock_snap_set_proceed_time        (MockSnap      *snap,
+                                                   const gchar   *proceed_time);
+
+void            mock_change_add_data              (MockChange    *change,
+                                                   JsonNode      *data);
+
+void            mock_change_set_kind              (MockChange    *change,
+                                                   const gchar   *kind);
+MockNotice *    mock_snapd_add_notice             (MockSnapd     *snapd,
+                                                   const gchar   *id,
+                                                   const gchar   *key,
+                                                   const gchar   *type);
+
+void            mock_notice_set_user_id           (MockNotice    *notice,
+                                                   const gchar   *user_id);
+
+void            mock_notice_set_dates             (MockNotice    *notice,
+                                                   GDateTime     *first_occurred,
+                                                   GDateTime     *last_occurred,
+                                                   GDateTime     *last_repeat,
+                                                   int            occurrences);
+
+void            mock_notice_set_expire_after      (MockNotice    *notice,
+                                                   const gchar   *expire_after);
+
+void            mock_notice_set_repeat_after      (MockNotice    *notice,
+                                                   const gchar   *repeat_after);
+
+void            mock_notice_add_data_pair         (MockNotice    *notice,
+                                                   const gchar   *entry,
+                                                   const gchar   *data);
+
+gchar *         mock_snapd_get_notices_parameters (MockSnapd     *snapd);
+
 G_END_DECLS
 
 #endif /* __MOCK_SNAPD_H__ */
